import json
import logging
import urllib
import ssl
from flask import Response, send_from_directory, Blueprint, request, abort
from barix.web.uci import getValueOfUci

from auth import token_required

bp = Blueprint('bp', __name__)
mainUrl = "http://127.0.0.1:8765"
DOWNLOAD_DIRECTORY = "/mnt/data/prerecorded_messages/"

log = logging.getLogger('flask-backend')


# @bp.route('/')
# def hello_world():
#     return "Hello World"


@bp.route('/groups', methods=['GET'])
@token_required
def get_group():
    try:
        log.info("Received: {} {}".format(request.method, request.full_path))
        request_headers = {}
        redirectionURL = mainUrl + "/groups"
        for (key, val) in request.headers:
            if key.lower() in ["content-length", "connection"]:
                continue
            request_headers[key] = val
        context = None
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        reqs = urllib.request.Request(url=redirectionURL, headers=request_headers, method=request.method)
        res = urllib.request.urlopen(reqs, context=context)
        data = res.read()
        enc = res.headers.get_content_charset('utf-8')
        payload = data.decode(enc)
        response_headers = {}
        for (key, value) in res.headers.items():
            if key.lower() in ["content-length", "connection"]:
                continue
            response_headers[key] = value
        log.info(
            "Sending Response of {} {} --> Headers: {} Status Code: {} Body: {}".format(request.method,
                                                                                        request.full_path,
                                                                                        response_headers, res.status,
                                                                                        payload))
        return buildResponse(response_headers, res.status, payload)
    except urllib.error.HTTPError as e:
        log.error(e)
        return buildResponse(e.headers, e.code)
    except Exception as e:
        log.error(e)
        abort(404)


@bp.route('/user', methods=['GET'])
@token_required
def get_user():
    try:
        token = request.args['token']
        log.info("Received: {} {}".format(request.method, request.full_path))
        with open("/barix/apps/icpgw/users_backup.json") as file:
            mydict = json.load(file)
            payload = None
            statusCode = None
            for key in mydict:
                if mydict[key]["secret"] == token:
                    name = mydict[key]["name"]
                    payload = json.dumps({"name": name})
                    statusCode = 200
        if payload is None:
            statusCode = 404

        headers = {'Content-type': 'application/json'}
        log.info("Sending Response of {} {} --> Headers: {} Status Code: {} Body: {}".format(request.method,
                                                                                             request.full_path, headers,
                                                                                             statusCode,
                                                                                             payload))
        return Response(response=payload, status=statusCode, headers=headers)
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/sample-rate', methods=['GET'])
@token_required
def get_sample_rate():
    try:
        log.info("Received: {} {}".format(request.method, request.full_path))
        sample_rate = getValueOfUci('icpgw', 'barp', 'sample_rate')
        headers = {'Content-type': 'application/json'}
        log.info("Sending Response of {} {} --> Headers: {} Status Code: 200 Body: {}".format(request.method,
                                                                                              request.full_path,
                                                                                              headers, sample_rate))
        return Response(response=sample_rate, status=200, headers={'Content-type': 'application/json'})
    except Exception as e:
        log.error(e)
        return Response(status=500)


@bp.route('/messages', methods=['GET'])
@token_required
def list_all_messages():
    try:
        log.info("Received: {} {}".format(request.method, request.full_path))
        with open("/barix/apps/icpgw/audio_files_backup.json") as file:
            mydict = json.load(file)
            payload = None
            payload = json.dumps(mydict)
            statusCode = 200
        if payload is None:
            statusCode = 404

        headers = {'Content-type': 'application/json'}
        log.info("Sending Response of {} {} --> Headers: {} Status Code: {} Body: {}".format(request.method,
                                                                                             request.full_path, headers,
                                                                                             statusCode,
                                                                                             payload))
        return Response(response=payload, status=statusCode, headers=headers)
    except Exception as e:
        log.error(e)
        return Response(status=500)


def iterform(multidict):
    for key in multidict.keys():
        for value in multidict.getlist(key):
            yield (key.encode("utf8"), value.encode("utf8"))


@bp.route('/get-audio', methods=['POST', 'PUT'])
@token_required
def get_audio():
    try:
        log.info("Received: {} {} {}".format(request.method, request.full_path, request.data))
        request_headers = {}
        redirectionURL = mainUrl + "/playfile"
        for (key, val) in request.headers:
            if key.lower() in ["content-length", "connection"]:
                continue
            request_headers[key] = val
        context = None
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        reqs = urllib.request.Request(url=redirectionURL, data=request.data, headers=request_headers,
                                      method=request.method)
        res = urllib.request.urlopen(reqs, context=context)
        data = res.read()
        enc = res.headers.get_content_charset('utf-8')
        payload = data.decode(enc)
        response_headers = {}
        for (key, value) in res.headers.items():
            if key.lower() in ["content-length", "connection"]:
                continue
            response_headers[key] = value
        log.info("Sending Response of {} {} --> Headers: {} Status Code: {} Body: {}".format(request.method,
                                                                                             request.full_path,
                                                                                             response_headers,
                                                                                             res.status, payload))
        return buildResponse(response_headers, res.status, payload)
    except urllib.error.HTTPError as e:
        log.error(e)
        return buildResponse(e.headers, e.code)
    except Exception as e:
        log.error(e)
        abort(404)


@bp.route('/paging', methods=['POST', 'PUT'])
@token_required
def paging():
    try:
        if request.method == "POST":
            log.info("Received: {} {} {}".format(request.method, request.full_path, request.data))
        else:
            log.info("Received: {} {}".format(request.method, request.full_path))
        request_headers = {}
        redirectionURL = mainUrl + "/paging"
        for (key, val) in request.headers:
            if key.lower() in ["content-length", "connection"]:
                continue
            request_headers[key] = val
        context = None
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        # Documentation urlopen(Method,URL,Body,Headers,ªªurlopen_kw)
        reqs = urllib.request.Request(url=redirectionURL, data=request.data, headers=request_headers,
                                      method=request.method)
        #        reqs.add_header('Content-Type', 'application/json')
        res = urllib.request.urlopen(reqs, context=context)
        response_headers = {}
        for (key, value) in res.headers.items():
            if key.lower() in ["content-length", "connection"]:
                continue
            response_headers[key] = value
        log.info("Sending Response of {} {} --> Headers: {} Status Code: {} ".format(request.method,
                                                                                     request.full_path,
                                                                                     response_headers,
                                                                                     res.status))
        return buildResponse(response_headers, res.status)
    except urllib.error.HTTPError as e:
        log.error(e)
        data = e.read()
        enc = e.headers.get_content_charset('utf-8')
        payload = data.decode(enc)
        log.info("Sending Response of {} {} --> Headers: {} Status Code: {} Response: {}".format(request.method,
                                                                                                 request.full_path,
                                                                                                 e.headers,
                                                                                                 e.code,
                                                                                                 payload))
        return buildResponse(e.headers, e.code, payload)
    except Exception as e:
        log.error(e)
        abort(404)


def buildResponse(headers, status, payload=None):
    response_headers = {}

    for (key, value) in headers.items():
        if key.lower() in ["content-length", "connection"]:
            continue
        response_headers[key] = value
    if payload is None:
        return Response(status=status, headers=response_headers)
    else:
        return Response(response=payload, status=status, headers=response_headers)


@bp.route('/recording', methods=['POST', 'PUT'])
@token_required
def recording():
    try:
        if request.method == "POST":
            log.info("Received: {} {} {}".format(request.method, request.full_path, request.data))
        else:
            log.info("Received: {} {}".format(request.method, request.full_path))
        request_headers = {}
        redirectionURL = mainUrl + "/recording"
        for (key, val) in request.headers:
            if key.lower() in ["content-length", "connection"]:
                continue
            request_headers[key] = val
        context = None
        context = ssl.create_default_context()
        context.check_hostname = False
        context.verify_mode = ssl.CERT_NONE
        # Documentation urlopen(Method,URL,Body,Headers,ªªurlopen_kw)
        reqs = urllib.request.Request(url=redirectionURL, data=request.data, headers=request_headers,
                                      method=request.method)
        res = urllib.request.urlopen(reqs, context=context)
        response_headers = {}
        for (key, value) in res.headers.items():
            if key.lower() in ["content-length", "connection"]:
                continue
            response_headers[key] = value
        log.info("Sending Response of {} {} --> Headers: {} Status Code: {}".format(request.method, request.full_path,
                                                                                    response_headers, res.status))
        return buildResponse(response_headers, res.status)
    except urllib.error.HTTPError as e:
        log.error(e)
        return buildResponse(e.headers, e.code)
    except Exception as e:
        log.error(e)
        abort(404)


@bp.route('/play-audio/<filename>', methods=['GET'])
@token_required
def get_files(filename):
    try:
        log.info("Received: {} {}".format(request.method, request.full_path))
        log.info("Sending Response: {}".format(DOWNLOAD_DIRECTORY + filename))
        return send_from_directory(DOWNLOAD_DIRECTORY, filename, as_attachment=True)
    except FileNotFoundError as e:
        log.error(e)
        abort(404)
