import argparse
import signal
import time
from threading import Thread

import shared_variables
from http_server import run_server, stop_server
from utils import get_logger
import constants

logger = get_logger("main")
listen_status_thread = None


def signal_handler(signum, frame):
    global listen_status_thread
    logger.info("Terminating application...")
    shared_variables.kill_playfile_thread = constants.YES
    logger.info(f"Listen status thread: {listen_status_thread}")
    if listen_status_thread is not None:
        listen_status_thread.join(timeout=1)
        listen_status_thread = None
        logger.info("ListenStatus thread stopped")

    shared_variables.pmaster.exit()
    stop_server()

    logger.info("Device stopped")
    exit()


def main():
    global listen_status_thread
    parser = argparse.ArgumentParser(
        description="Script to start backend Paging server."
    )
    parser.add_argument(
        "--port", type=int, default=8765, help="local port at which server listens."
    )
    args = parser.parse_args()

    try:
        shared_variables.initialize()
        logger.info("Shared variables initialization successful!!!")
    except Exception as e:
        logger.error(f"Shared variables initialization failed :{e}")
        return

    # Run the WD before initialisation of the variables
    logger.info("Starting backup WD...")
    backup_files_thread = Thread(target=shared_variables.backup_files.run)
    backup_files_thread.setName("BackupFiles")
    backup_files_thread.daemon = True
    backup_files_thread.start()
    logger.info("Done!")

    logger.info("Checking for audio file changes...")
    shared_variables.audio_processor.run()
    logger.info("Done!")

    listen_status_thread = Thread(target=shared_variables.pmaster.listen_status_message)
    listen_status_thread.setName("ListenStatus")
    listen_status_thread.daemon = True
    listen_status_thread.start()

    # NOTE: UNCOMMENT if the gateway should report its status as well.
    # master_status_thread = Thread(target=shared_variables.pmaster.send_status_message)
    # master_status_thread.daemon = True
    # master_status_thread.start()

    # Staring of the Server
    try:
        run_server(port=args.port)
    except Exception as e:
        logger.error(f"Paging server failed {e}")

    # shared_variables.pmaster.exit()
    # listen_status_thread.join()


if __name__ == "__main__":
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)
    main()
