import alsaaudio
from paging_master import PagingMaster
from utils import uci_to_config_dict
from groups import Groups
from users import Users
from audio_files import AudioFiles  # Anton
from audio_files_processor import AudioFilesProcessor  # Anton
from record_service import MessageRecordService  # Anton
from backup_settings_wd import BackupSettingsWD  # Anton
import constants  # Anton
from utils import get_logger


logger = get_logger("SharedVariables")


def initialize():
    global config
    global pmaster
    global last_message_type
    global active_browser
    global TIMEOUT
    global groups
    global users
    global audio_files  # Anton
    global audio_processor  # Anton
    global playfile_state  # Anton
    global delete_processed_files  # Anton
    global downloading_recorded_message  # Anton
    global paging_message_q
    global paging_message_q_buff_msg
    global paging_state
    global record_service
    global backup_files
    global local_alsa_player
    global last_client
    global user_token_length
    global active_user
    global active_user_uuid
    global first_fast_packets

    # Thread handles
    global local_playback_queue_process
    global local_playback_process
    global paging_message_q_thread
    global send_page_group_thread
    global playfile_group_thread
    global playfile_playing_thread

    # Flags
    global exit_now
    global end_operation
    global kill_playfile_thread  # Anton
    global kill_local_playback_thread  # Anton

    global local_playback_msg_queue
    global last_click_time

    playfile_group_thread = None
    playfile_playing_thread = None

    last_click_time = -1
    exit_now = False
    local_playback_queue_process = None
    local_playback_msg_queue = None
    end_operation = True  # Only used to stop local playback thread
    send_page_group_thread = None

    active_user = None
    active_user_uuid = None

    last_client = None
    groups = Groups()
    users = Users()

    paging_state = constants.IDLE
    paging_message_q = None
    paging_message_q_thread = None
    paging_message_q_buff_msg = 0
    first_fast_packets = 0
    local_playback_process = None
    playfile_state = constants.IDLE  # Anton
    delete_processed_files = constants.YES  # Anton
    kill_playfile_thread = constants.NO  # Anton
    downloading_recorded_message = constants.YES  # Anton
    kill_local_playback_thread = constants.NO  # Anton
    audio_processor = AudioFilesProcessor()  # Anton
    audio_files = AudioFiles()  # Anton

    user_token_length = len(users.new_secret)
    config = uci_to_config_dict()

    record_service = MessageRecordService()  # Anton

    TIMEOUT = config["browser_timeout"]  # seconds
    active_browser = {"time": None, "id": ""}
    pmaster = PagingMaster(config["paging_master_id"])
    last_message_type = 8

    backup_files = BackupSettingsWD(["*_backup.json"])  # Anton

    local_alsa_player = alsaaudio.PCM(
        alsaaudio.PCM_PLAYBACK, alsaaudio.PCM_NORMAL, config["audio_device"]["playback"]  # plug:master1
    )
    local_alsa_player.setchannels(1)
    local_alsa_player.setperiodsize(32)  # 2972
    local_alsa_player.setrate(config["sample_rate"])
    local_alsa_player.setformat(alsaaudio.PCM_FORMAT_S16_LE)


def get_new_alsa_player(codec):
    if codec == "MULAW":
        player_format = alsaaudio.PCM_FORMAT_MU_LAW
    elif codec == "ALAW":
        player_format = alsaaudio.PCM_FORMAT_A_LAW
    else:
        player_format = alsaaudio.PCM_FORMAT_S16_LE

    alsa_player = alsaaudio.PCM(
        alsaaudio.PCM_PLAYBACK, alsaaudio.PCM_NORMAL, config["audio_device"]["playback"]  # plug:master1
    )
    alsa_player.setchannels(1)
    alsa_player.setperiodsize(32)  # 2972
    alsa_player.setrate(config["sample_rate"])
    alsa_player.setformat(player_format)
    return alsa_player


def write_to_local_playback(data, live_paging=False, alsa_player=None, codec="PCM"):
    # logger.info(f"kill_local_playback_thread: {kill_local_playback_thread}")
    if alsa_player is None:
        # player = get_new_alsa_player(codec=codec)
        player = local_alsa_player
    else:
        player = alsa_player

    if live_paging:
        try:
            player.write(data)
        except Exception as e:
            print(f"player.write(data): {e}")
    else:
        global kill_local_playback_thread
        logger.info("Starting local playback")
        kill_local_playback_thread = constants.NO

        data_block_size = 128
        leftover = len(data) % data_block_size
        count = 0
        try:
            for i in range(len(data) // data_block_size):
                if not kill_local_playback_thread:
                    # logger.info(f"Adding packet: {i}, kill_local_playback_thread: {kill_local_playback_thread}")
                    player.write(data[i * data_block_size: (i + 1) * data_block_size])
                    count = count + data_block_size
                else:
                    break
            if leftover and not kill_local_playback_thread:
                logger.info("Sending leftover")
                player.write(data[len(data) - leftover:])
                count = count + leftover
            logger.info(f"Played locally: {count / 2}, out of {len(data) / 2} bytes")
            logger.info("Finished local playback")
        except Exception as e:
            print(f"player.write(data): {e}")


def kill_alsa_player():
    global kill_local_playback_thread
    kill_local_playback_thread = constants.YES


def set_last_click_time(new_time):
    logger.info("Button click")
    global last_click_time
    if new_time != last_click_time:
        last_click_time = new_time
