import logging
import os
import uuid

from .uci import getValueOfUci

from .constants import *

log = logging.getLogger('flask-backend')

'''
Generate a new session key
Session Key is used to validate logins.
A unique ID is generated and stored in the file SESSION_KEY_FILE_PATH.
'''
def generateNewSessionKey():
    sessionKey = uuid.uuid4().hex
    log.debug("New SESSION_KEY generated")
    try:
        f = open(SESSION_KEY_FILE_PATH, 'w')
        f.write(sessionKey)
        f.close()
    except Exception as e:
        log.error("Error while writing in file {}: {}".format(SESSION_KEY_FILE_PATH, e))
        raise e


'''
Get the session key
Session Key is used to validate logins.
Reads the file SESSION_KEY_FILE_PATH, which is where the session key is stored.
If the file doesn't exist or if it is empty, generates a new session key.
Returns the session key (string).
'''
def getSessionKey():
    #return uuid.uuid4().hex
    sessionKey = None
    # if file does not exist, create it
    if not os.path.isfile(SESSION_KEY_FILE_PATH):
        generateNewSessionKey()
    else: # file exists
        f = open(SESSION_KEY_FILE_PATH, 'r')
        sessionKey = f.read()
        if sessionKey == '': # file is empty
            generateNewSessionKey()
    return sessionKey

from functools import wraps
from flask import session, Response

'''
Check if login is required and, if so, if user is logged in.
If login is not required or if it is required and the user is already logged in, allows to proceed. Otherwise, a Non Authorized 401 HTTP Response is returned.
To check if the user is logged in, compared the sessionID from the request to the sessionID stored. If none is provided in the request or if they don't match, then login is invalid. 
'''
def login_required(f):
    @wraps(f)
    def decorated_function(*args, **kwargs):
        # log.debug("Session Key: {}".format(getSessionKey()))
        # log.debug("Session: {}".format(session))
        try:
            login_uci = getValueOfUci('httpd', 'webserver', 'password_set')
        except Exception:
            return Response(status=500)
        else:
            if login_uci == 'true':
                if 'sessionID' not in session or session['sessionID'] != getSessionKey():
                    return Response("Login required", status=401, headers={"Content-Type": "text/plain"})
                #elif session['sessionID'] != common.SESSION_KEY:
                #    return Response("Session expired", status=INVALID_LOGIN, headers={"Content-Type" : "text/plain"})
                else:
                    return f(*args, **kwargs)
            else:
                return f(*args, **kwargs)
    return decorated_function