let groupsList = {};
let sortedGroupsList = [];
let N_ROWS = 5;
let N_COLS = 3;
let pageIdx = 0;
let maxPageIdx;
let onClickCallback = null;

function sortGroupsByName(groupsReceived){
    let list = [];
    Object.keys(groupsReceived).forEach(groupID => {
        list.push({
            "id": groupID,
            "name": groupsReceived[groupID]["name"],
            "clients": groupsReceived[groupID]["clients"]
        })
    });
    list.sort((a, b) => {
        let firstToCompare = "";
        if (a.name === ""){
            firstToCompare = ("Group "+a.id.toString()).toLowerCase();
        } else {
            firstToCompare = a.name.toLowerCase();
        }
        let secondToCompare = "";
        if (b.name === ""){
            secondToCompare = ("Group "+b.id.toString()).toLowerCase();
        } else {
            secondToCompare = b.name.toLowerCase();
        }
        return firstToCompare.localeCompare(secondToCompare)
    });
    return list;
}

function sortGroupsById(groupsReceived){
    let list = [];
    Object.keys(groupsReceived).forEach(groupID => {
        list.push({
            "id": groupID,
            "name": groupsReceived[groupID]["name"],
            "clients": groupsReceived[groupID]["clients"]
        })
    });
    list.sort((a, b) => {
        return parseInt(a.id) - parseInt(b.id);
    });
    return list;
}

function calculateMaxPageIdx(){
    let max = Math.floor(sortedGroupsList.length/(N_ROWS*N_COLS)) - 1;
    let rest = sortedGroupsList.length % (N_ROWS*N_COLS)
    if (rest !== 0){
        max += 1;
    }
    maxPageIdx = max;
}

/**
 Show modal with group details like name and clients list.
 Depending on the mode, it is possible to edit its contents.

 Mode: string -> "read-only" | "edit" | "new"
 Group name: string
**/

function allClientsSelected() {
    let allChecked = true;
    for (let i = 1; i < 129; i++){
        let elemID = "checkbox-client-"+i.toString();
        const clientCheckboxElem = document.getElementById(elemID);
        if (!clientCheckboxElem.checked) {
            allChecked = false;
            break;
        }
    }
    return allChecked;
}

function buildClientCheckboxesTable(mode) {

    if (window.screen.width < 500){
        n_cols = 12;
    } else {
        n_cols = 16;
    }
    const modalGroupTableElem = document.getElementById("group-clients-table");
    modalGroupTableElem.innerHTML="";
    modalGroupTableElem.setAttribute("cellspacing", "0");
    modalGroupTableElem.setAttribute("cellpadding", "0");
    modalGroupTableElem.setAttribute("class", "labelValue");

    let i = 0;
    while (i < 128){
        const tableRow = modalGroupTableElem.insertRow(-1);
        for (let j = 0; j < n_cols; j++){
            if ( i < 128 ){
                const tableCell = tableRow.insertCell(-1);
                const cellDivElem = document.createElement("div");
                cellDivElem.setAttribute("class", "client-checkbox-div");
                const checkBoxElem = document.createElement("input");
                checkBoxElem.setAttribute("type", "checkbox");
                checkBoxElem.setAttribute("id", "checkbox-client-"+(i+1).toString());
                checkBoxElem.setAttribute("value", (i+1).toString());
                checkBoxElem.onclick = function () {
                    const selectAllCheckbox = document.getElementById("selectAllCheckbox");
                    if (allClientsSelected()){
                        selectAllCheckbox.checked = true;
                    } else {
                        selectAllCheckbox.checked = false;
                    }
                }
                if (window.screen.width < 500){
                    checkBoxElem.setAttribute("style", "width: 40px; height: 40px;");
                } else {
                    checkBoxElem.setAttribute("style", "width: 25px; height: 25px;");
                }
                cellDivElem.appendChild(checkBoxElem);
                const checkBoxLabelElem = document.createElement("label");
                checkBoxLabelElem.innerText= (i+1).toString();
                if (window.screen.width < 500){
                    checkBoxLabelElem.setAttribute("style", "font-size:24px;");
                }
                cellDivElem.appendChild(checkBoxLabelElem);
                tableCell.appendChild(cellDivElem);
                i++;
            } else {
                break;
            }
        }
    }
}

function checkClientCheckboxes(clientsList) {
    clientsList.forEach(clientID => {
        const clientCheckboxElem = document.getElementById("checkbox-client-"+clientID.toString());
        clientCheckboxElem.checked = true;
    });
}

function openGroupDetailsModal(group, mode){
    buildModal("group-modal");
    showPopup("group-modal",false, false);

    const modalTitleElem = document.getElementById("group-modal-title");
    const groupNameInputElem = document.getElementById("group-name-input");

    // group name
    if (mode === "read-only"){
        //set modal title according to mode
        modalTitleElem.innerText = "Group Details";
        groupNameInputElem.value = groupNameInputElem.defaultValue = group.name;
        groupNameInputElem.placeholder = "Group "+group.id.toString();
    } else if (mode === "add") {
        //set modal title according to mode
        modalTitleElem.innerText = "New Group";
        groupNameInputElem.value = groupNameInputElem.defaultValue = "";
        groupNameInputElem.placeholder = "Add name";
    } else if (mode === "edit") {
        //set modal title according to mode
        modalTitleElem.innerText = "Edit Group";
        groupNameInputElem.value = groupNameInputElem.defaultValue = group.name;
        groupNameInputElem.placeholder = "Group "+group.id.toString();
    }

    if (window.screen.width < 500){
        groupNameInputElem.setAttribute("style", "font-size: 24px;");
        modalTitleElem.setAttribute("style", "font-size:40px;")
        const groupNameLabel = document.getElementById("group-name-label");
        groupNameLabel.setAttribute("style", "font-size: 24px;");
        const groupClientsLabel = document.getElementById("group-clients-label");
        groupClientsLabel.setAttribute("style", "font-size: 24px;");
    }


    //build client checkboxes table
    buildClientCheckboxesTable();
    if (mode === "edit" || mode === "read-only") checkClientCheckboxes(group.clients);

    //select all checkbox configuration
    const selectAllCheckbox = document.getElementById("selectAllCheckbox");
    if (window.screen.width < 500){
        selectAllCheckbox.setAttribute("style", "width: 30px; height: 30px;");
    } else {
        selectAllCheckbox.setAttribute("style", "width: 20px; height: 20px;");
    }
    if (allClientsSelected()){
        selectAllCheckbox.checked = true;
    } else {
        selectAllCheckbox.checked = false;
    }
    const selectAllSpan = document.getElementById("selectAllSpan");
    if (window.screen.width < 500){
        selectAllSpan.setAttribute("style", "font-size: 24px;");
    }

    //delete group btn configuration
    const deleteGroupBtn = document.getElementById("deleteGroupBtn");
    if (mode === "edit") {
        deleteGroupBtn.hidden = false;
        deleteGroupBtn.onclick = function () {
            removeGroup(group.id);
        }
        if (window.screen.width < 500){
            deleteGroupBtn.setAttribute("style", "font-size: 24px; margin-right: 100px; height: 40px;");
        }
    } else {
        deleteGroupBtn.hidden = true;
    }

    //submit and cancel buttons configuration
    if (mode === "edit" || mode === "add"){
        const submitBtn = document.getElementById("modal-submit-btn");
        submitBtn.hidden = false;
        if (mode === "edit") {
            submitBtn.onclick = function () {
                saveBtnOnClick(mode, group.id);
            }
        } else {
            submitBtn.onclick = function () {
                saveBtnOnClick(mode, null);
            }
        }
        if (window.screen.width < 500) {
            submitBtn.setAttribute("style", "font-size: 24px; height: 40px;");
        }
        const cancelBtn = document.getElementById("modal-cancel-btn");
        cancelBtn.hidden = false;
        if (window.screen.width < 500) {
            cancelBtn.setAttribute("style", "font-size: 24px; height: 40px;");
        }
    }
}

function selectAllCheckboxOnClick() {
    const selectAllCheckbox = document.getElementById("selectAllCheckbox");
    if (selectAllCheckbox.checked) {
        for (let i = 1; i < 129; i++){
            let elemID = "checkbox-client-"+i.toString();
            const clientCheckboxElem = document.getElementById(elemID);
            clientCheckboxElem.checked = true;
        }
    } else {
        for (let i = 1; i < 129; i++){
            let elemID = "checkbox-client-"+i.toString();
            const clientCheckboxElem = document.getElementById(elemID);
            clientCheckboxElem.checked = false;
        }
    }
}

function showGroupDetails(group) {
    openGroupDetailsModal(group, "read-only");
}

function editGroupDetails(group) {
    openGroupDetailsModal(group, "edit");
}

function newGroup() {
    openGroupDetailsModal(null, "add");
}

function removeGroup(groupID) {
    let groupName = groupsList[groupID]["name"];
    if (groupName === ""){
        groupName = "Group "+groupID.toString();
    }
    let r = confirm("Are you sure you wan't to delete Group \""+groupName+"\"?");
    if (r === true) {
        //OK
        // close modal
        closePopup("group-modal");
        // hide groups div
        commonToolsHideElemId('main-content');
        // show spinner
        commonToolsShowElemId('loading-spinner');

        // wait for the user to see the loading spinner
        setTimeout(function (){
            removeGroupRequest(groupID, groupName);
        }, 1500);
    } else {
    }
}

function handleHTTPSuccessResponse(msg) {

    // hide groups div
    commonToolsHideElemId('loading-spinner');
    // show spinner
    commonToolsShowElemId('main-content');
    // reload data
    createGroupsList();
    const groupDiv = document.getElementById('groups-section');
    if (groupDiv) {
        groupDiv.innerHTML = "";
        const mediaInternal = renderGroupDiv();
        groupDiv.appendChild(mediaInternal);
        if (sortedGroupsList.length !== 0) renderGroupsTable();
    } else {
        console.error("ERROR! Element can't be deployed! Please, clear browser cache and reload.");
    }
    // show notification
    notify.notify({message: msg, level:'good'});
}

function submitChanges(mode, groupID, groupName, clientsList) {
    // close modal
    closePopup("group-modal");
    // hide groups div
    commonToolsHideElemId('main-content');
    // show spinner
    commonToolsShowElemId('loading-spinner');

    // wait for the user to see the loading spinner
    setTimeout(function (){
        // make request
        if (mode === "add") {
            addGroupRequest(groupName, clientsList);
        } else if (mode === "edit"){
            editGroupRequest(groupID, groupName, clientsList);
        }
    }, 1500);
}

function saveBtnOnClick(mode, groupID) {
    let groupName = document.getElementById("group-name-input").value;
    let clientsList = [];
    for (let i = 1; i < 129; i++){
        let elemID = "checkbox-client-"+i.toString();
        const clientCheckboxElem = document.getElementById(elemID);
        if (clientCheckboxElem.checked) {
            clientsList.push(i);
        }
    }

    //check if name is valid
    if (groupName.replace(/\s+/g, '') === ""){
        groupName = "";
    }
    //check if clients are valid
    if (clientsList.length === 0){
        notify.notify({message:'You must select at least 1 client!', level:'danger'});
    } else {
        submitChanges(mode, groupID, groupName, clientsList);
    }

}

function clearGroupsTable() {
    const mediaTable = document.getElementById("groups-table");
    mediaTable.innerHTML="";
}

function renderGroupsTable(){

    clearGroupsTable();

    const mediaTable = document.getElementById("groups-table");
    mediaTable.setAttribute("cellspacing", "0");
    mediaTable.setAttribute("cellpadding", "0");
    mediaTable.setAttribute("class", "labelValue");

    for (let i = 0; i < N_ROWS; i++){
        let stop = false;
        const tableRow = mediaTable.insertRow(-1);
        for (let j = 0; j < N_COLS; j++){
            if ((i * N_COLS + j + pageIdx * N_ROWS * N_COLS) < sortedGroupsList.length){
                let group = sortedGroupsList[i * N_COLS + j + pageIdx * N_ROWS * N_COLS];
                const tableCell = tableRow.insertCell(-1);
                const groupBtn = document.createElement("button");
                groupBtn.setAttribute("class", "group-button");
                groupBtn.setAttribute("id", "group-button-"+group.id.toString());
                groupBtn.innerText = group.name;
                if (group.name === ""){
                    groupBtn.innerText = "Group "+group.id.toString();
                }
                // show more
                groupBtn.onclick = function () {
                    onClickCallback(group);
                }
                tableCell.appendChild(groupBtn);
            } else {
                stop = true;
                break;
            }
        }
        if (stop) break;
    }
}

function changePageIdx(direction) {
    if (direction === "right"){
        if (pageIdx < maxPageIdx) {
            pageIdx+=1;
            renderGroupsTable();
            document.getElementById("left-arrow").disabled = false;
            if (pageIdx === maxPageIdx) document.getElementById("right-arrow").disabled = true;
        } else {
        }
    } else {
        if (pageIdx > 0){
            pageIdx-=1;
            renderGroupsTable();
            document.getElementById("right-arrow").disabled = false;
            if (pageIdx === 0) document.getElementById("left-arrow").disabled = true;
        } else {
        }
    }
}

function renderGroupDiv() {

    if (sortedGroupsList.length === 0) {
        const noFilesDivElem = document.createElement("div");
        noFilesDivElem.setAttribute("class", "section2");
        noFilesDivElem.setAttribute("style", "font-size: 13px; font-style: italic; padding: 4px; color: #888888; text-align: center; margin-top: 10%;");
        noFilesDivElem.innerText = "No groups configured";
        return noFilesDivElem;
    }

    // create arrow left div
    const arrowLeftDivElem = document.createElement("div");
    arrowLeftDivElem.setAttribute("class", "arrow-div");
    arrowLeftDivElem.setAttribute("style", "margin-right:2%");
    const arrowLeftBtn = document.createElement("button");
    arrowLeftBtn.setAttribute("class", "arrow-btn");
    arrowLeftBtn.setAttribute("id", "left-arrow");
    arrowLeftBtn.disabled = true;
    const arrowLeftElem = document.createElement("i");
    arrowLeftElem.onclick = function () {
        changePageIdx("left");
    }
    arrowLeftElem.setAttribute("class", "fas fa-caret-left"); //font-awesome
    arrowLeftBtn.appendChild(arrowLeftElem)
    arrowLeftDivElem.appendChild(arrowLeftBtn);

    // create arrow right div
    const arrowRightDivElem = document.createElement("div");
    arrowRightDivElem.setAttribute("class", "arrow-div");
    arrowRightDivElem.setAttribute("style", "margin-left:2%");
    const arrowRightBtn = document.createElement("button");
    arrowRightBtn.setAttribute("class", "arrow-btn");
    arrowRightBtn.setAttribute("id", "right-arrow");
    if (pageIdx === maxPageIdx){
        arrowRightBtn.disabled = true;
    }
    const arrowRightElem = document.createElement("i");
    arrowRightElem.onclick = function () {
        changePageIdx("right");
    }
    arrowRightElem.setAttribute("class", "fas fa-caret-right"); //font-awesome

    arrowRightBtn.appendChild(arrowRightElem);
    arrowRightDivElem.appendChild(arrowRightBtn);


    // create table div with groups
    const mediaTable = document.createElement("table");
    mediaTable.setAttribute("id", "groups-table");

    const mainTable = document.createElement("table");
    const tableRow = mainTable.insertRow(-1);

    tableRow.setAttribute("align", "center");
    tableRow.setAttribute("border", "0");
    tableRow.setAttribute("cellspacing", "0");
    tableRow.setAttribute("cellpadding", "0");

    const tableLeftCell = tableRow.insertCell(-1);
    tableLeftCell.appendChild(arrowLeftDivElem);
    const tableMiddleCell = tableRow.insertCell(-1);
    tableMiddleCell.appendChild(mediaTable);
    const tableRightCell = tableRow.insertCell(-1);
    tableRightCell.appendChild(arrowRightDivElem);

    return mainTable;

}

/**
 * Render the existing groups
 */
function createGroupsList() {
    groupsList = getListOfGroups();
    //console.log(groupsList);
    //sortedGroupsList = sortGroupsByName(groupsList);
    sortedGroupsList = sortGroupsById(groupsList);
    //console.log(sortedGroupsList);
    calculateMaxPageIdx();
}

function loadGroups(btnClickFunction) {
    onClickCallback = btnClickFunction;
    if (window.screen.width < 500){
        const newGroupBtn = document.getElementById("newGroupBtn");
        if (newGroupBtn) newGroupBtn.setAttribute("style", "width: 170px; height: 60px; font-size: 24px;")
    }
    const groupDiv = document.getElementById('groups-section');
    if (groupDiv) {
        createGroupsList();
        const mediaInternal = renderGroupDiv();
        groupDiv.appendChild(mediaInternal);
        if (sortedGroupsList.length !== 0) renderGroupsTable();
    } else {
        console.error("ERROR! Element can't be deployed! Please, clear browser cache and reload.");
    }
}
