// Endpoint to obtain the configuration values from UCI in JSON format
const USERS_ENDPOINT = "/users";

let usersList = {};
let N_ROWS = 10;

/**
 * Obtain a sources config using ajax. NOTE: this method is synchronous.
 */
async function getListOfUsers() {
    let usersReceivedList = {};
    $.ajax({
        async: false,
        type: "GET",
        url: USERS_ENDPOINT,
        success: function (result, status, xhr) {
            usersReceivedList = result;
        },
        error: function (xhr, status, error) {
            console.log("It was not possible to get the existing users");
            usersReceivedList = null;
        }
    });
    return usersReceivedList;
}

/**
 Show modal with user details like name and clients list.
 Depending on the mode, it is possible to edit its contents.

 Mode: string -> "read-only" | "edit" | "new"
 User name: string
**/
function openUserDetailsModal(userID, user, mode){
    buildModal("user-modal");
    showPopup("user-modal",false, false);

    const modalTitleElem = document.getElementById("user-modal-title");
    const userNameInputElem = document.getElementById("user-name-input");
    userNameInputElem.placeholder = "Username";
    const userEmailInputElem = document.getElementById("user-email-input");
    userEmailInputElem.placeholder = "Email address";

    if (mode === "add") {
        //set modal title according to mode
        modalTitleElem.innerText = "Create New User";
        userNameInputElem.value = userNameInputElem.defaultValue = "";
        userEmailInputElem.value = userEmailInputElem.defaultValue = "";
    } else if (mode === "edit") {
        //set modal title according to mode
        modalTitleElem.innerText = "Edit User";
        userNameInputElem.value = userNameInputElem.defaultValue = user.name;
        userNameInputElem.placeholder = "User "+userID.toString();
        userEmailInputElem.value = userEmailInputElem.defaultValue = user.email;
    }

    //submit and cancel buttons configuration
    if (mode === "edit" || mode === "add"){
        const submitBtn = document.getElementById("modal-submit-btn");
        if (mode === "edit") {
            submitBtn.onclick = function () {
                saveBtnOnClick(mode, userID);
            }
        } else {
            submitBtn.onclick = function () {
                saveBtnOnClick(mode, null);
            }
        }
    }
}

function selectOrDeselectAll() {
    const checkboxAllElem = document.getElementById("checkbox-all");
    Object.keys(usersList).forEach((id) => {
        if (id !== "0"){
            let elemID = "checkbox-user-"+id.toString();
            const userCheckboxElem = document.getElementById(elemID);
            if (checkboxAllElem.checked === true){
                userCheckboxElem.checked = true;
            } else {
                userCheckboxElem.checked = false;
            }   
        }  
    });
}

function selectOrDeselectCheckboxAll() {
    const checkboxAllElem = document.getElementById("checkbox-all");
    let shouldSelectCheckboxAll = true;
    for (i = 0; i < Object.keys(usersList).length; i++){
        let id = Object.keys(usersList)[i];
        let elemID = "checkbox-user-"+id.toString();
        const userCheckboxElem = document.getElementById(elemID);
        if (!userCheckboxElem.checked){
            shouldSelectCheckboxAll = false;
            break;
        }
    }
    if (shouldSelectCheckboxAll === true){
        checkboxAllElem.checked = true;
    } else {
        checkboxAllElem.checked = false;
    }
}

function editUserDetails(userID, user) {
    openUserDetailsModal(userID, user, "edit");
}

function newUser() {
    openUserDetailsModal(null, null, "add");
}

function deleteUsers() {
    listOfUsersToDelete = [];

    Object.keys(usersList).forEach((id) => {
        let elemID = "checkbox-user-"+id.toString();
        const userCheckboxElem = document.getElementById(elemID);
        if (userCheckboxElem.checked === true){
            listOfUsersToDelete.push(id);
        } 
    });

    if (listOfUsersToDelete.length === 0){
        alert("Please select one or more users to delete them.");
    } else {
        let r = confirm("Are you sure you want to delete selected users?");
        if (r === true) {
            //OK
            // hide users div
            commonToolsHideElemId('main-content');
            // show spinner
            commonToolsShowElemId('loading-spinner');

            // wait for the user to see the loading spinner
            setTimeout(function (){
                //console.log(listOfUsersToDelete);
                $.ajax({
                    type: "DELETE",
                    url: USERS_ENDPOINT,
                    data: JSON.stringify({ "id": listOfUsersToDelete }),
                    success: function(result, status, xhr){
                        handleHTTPResponse('Selected users deleted successfully', 'good');
                    },
                    error: function (xhr, status, error) {
                        handleHTTPResponse('It was not possible to delete selected users.', 'danger');
                    }
                });
            }, 1500);
        } else {
        }
    }
}

async function handleHTTPResponse(notify_msg, notify_level) {

    // hide users div
    commonToolsHideElemId('loading-spinner');
    // show spinner
    commonToolsShowElemId('main-content');
    // reload data
    const userDiv = document.getElementById('users-section');
    if (userDiv) {
        usersList = await getListOfUsers();
        const mediaInternal = renderUserDiv();
        userDiv.appendChild(mediaInternal);
        if (Object.keys(usersList).length !== 0) renderUsersTable();
        paginate('#users-table',N_ROWS);
    } else {
        console.error("ERROR! Element can't be deployed! Please, clear browser cache and reload.");
    }
    // show notification
    notify.notify({message: notify_msg, level: notify_level});
}

function createUserRequest(userName, userEmail) {
    $.ajax({
        type: "POST",
        url: USERS_ENDPOINT,
        data: JSON.stringify({"name":  userName, "email": userEmail}),
        success: function(result, status, xhr){
            let msg = "";
            if (userName === ""){
                msg = 'Configuration of User '+ result["id"] +' saved successfully';
            } else {
                msg = 'Configuration of user \"'+ userName +'\" saved successfully';
            }
            handleHTTPResponse(msg, 'good');
        },
        error: function (xhr, status, error) {
            if (xhr["status"] === 400 && xhr["responseJSON"] === "Improper Request"){
                msg = 'Maximum number of users reached.';    
            } else {
                msg = 'It was not possible to save the user configuration.';    
            }
            handleHTTPResponse(msg, 'danger');
        }
    });
}

function editUserRequest(userID, userName, userEmail) {
    $.ajax({
        type: "UPDATE",
        url: USERS_ENDPOINT,
        data: JSON.stringify({"id": userID, "name":  userName, "email": userEmail}),
        success: function(result, status, xhr){
            let msg = "";
            if (userName === ""){
                msg = 'Configuration of User '+ userID +' saved successfully';
            } else {
                msg = 'Configuration of user \"'+ userName +'\" saved successfully';
            }
            handleHTTPResponse(msg, 'good');
        },
        error: function (xhr, status, error) {
            handleHTTPResponse('It was not possible to save the user configuration.', 'danger');
        }
    });
}

function submitChanges(mode, userID, userName, userEmail) {
    // close modal
    closePopup("user-modal");
    // hide users div
    commonToolsHideElemId('main-content');
    // show spinner
    commonToolsShowElemId('loading-spinner');

    // wait for the user to see the loading spinner
    setTimeout(function (){
        // make request
        if (mode === "add") {
            createUserRequest(userName, userEmail);
        } else if (mode === "edit"){
            editUserRequest(userID, userName, userEmail);
        }
    }, 1500);
}

function saveBtnOnClick(mode, userID) {
    let userName = document.getElementById("user-name-input").value;
    let userEmail = document.getElementById("user-email-input").value;

    //check if name is valid
    if (userName.replace(/\s+/g, '') === "" || (userID !== null && userName === "User "+userID)){
        userName = "";
    }
    //check if email is valid
    if (userEmail.replace(/\s+/g, '') === ""){
        userEmail = "";
    }
    submitChanges(mode, userID, userName, userEmail);

}

function downloadUsersList() {

    let csvContent = "data:text/csv;charset=utf-8,";

    csvContent += "ID,Name,Email,Access URL\r\n";

    Object.keys(usersList).forEach( id => {
        csvContent += id.toString() + "," + usersList[id]["name"].toString() + "," + usersList[id]["email"].toString() + "," + window.location.origin + "/app/paging_users_frame.html?token="+usersList[id]["secret"].toString() + "\r\n";
    });

    var encodedUri = encodeURI(csvContent);
    var link = document.createElement("a");
    link.setAttribute("href", encodedUri);
    link.setAttribute("download", "users.csv");
    document.body.appendChild(link);

    link.click();
}


function renderUsersTable(){

    const mediaTable = document.getElementById("users-table");

    mediaTable.innerHTML = "";

    mediaTable.setAttribute("cellspacing", "0");
    mediaTable.setAttribute("cellpadding", "0");

    // create table headers
    const header = document.createElement("thead");
    mediaTable.appendChild(header);

    let headerCell = document.createElement("th");
    header.appendChild(headerCell);

    let checkboxAllElem = document.createElement("input");
    checkboxAllElem.setAttribute("type", "checkbox");
    checkboxAllElem.setAttribute("id", "checkbox-all");
    checkboxAllElem.onclick = function () {
        selectOrDeselectAll();
    }
    headerCell.appendChild(checkboxAllElem);


    headerCell = document.createElement("th");
    headerCell.innerHTML = 'Name';
    header.appendChild(headerCell);

    headerCell = document.createElement("th");
    headerCell.innerHTML = 'E-mail';
    header.appendChild(headerCell);

    headerCell = document.createElement("th");
    headerCell.innerHTML = 'Access URL';
    header.appendChild(headerCell);

    headerCell = document.createElement("th");
    header.appendChild(headerCell);

    let body = document.createElement("tbody");
    mediaTable.appendChild(body);


    Object.keys(usersList).forEach( id => {
        let tableRow = body.insertRow(-1);
        
        // create cell for checkbox
        let tableCell = tableRow.insertCell(-1);
        let checkBoxElem = document.createElement("input");
        checkBoxElem.setAttribute("type", "checkbox");
        checkBoxElem.setAttribute("id", "checkbox-user-"+id.toString());
        checkBoxElem.onclick = function (){
            selectOrDeselectCheckboxAll();
        }
        if (id === "0"){
            checkBoxElem.disabled=true;
        }
        tableCell.appendChild(checkBoxElem);

        // create cell for name
        tableCell = tableRow.insertCell(-1);
        labelElem = document.createElement("p");
        if (usersList[id]["name"] === ""){
            labelElem.innerHTML = "User "+id.toString();
            labelElem.setAttribute("style", "font-style: italic");
        } else {
            labelElem.innerHTML = usersList[id]["name"].toString();
        }
        tableCell.appendChild(labelElem);

        // create cell for email
        tableCell = tableRow.insertCell(-1);
        labelElem = document.createElement("p");
        labelElem.innerHTML = usersList[id]["email"].toString();
        tableCell.appendChild(labelElem);

        // create cell for URL
        tableCell = tableRow.insertCell(-1);
        labelElem = document.createElement("a");
        let path = "/app/paging_users_frame.html?token="+usersList[id]["secret"].toString();
        labelElem.innerHTML = path;
        labelElem.setAttribute("href", window.location.protocol + "//" + window.location.hostname + path);
        labelElem.setAttribute("target", "_blank");
        tableCell.appendChild(labelElem);

        // create cell for Action Buttons
        tableCell = tableRow.insertCell(-1);
        iconElem = document.createElement("i");
        iconElem.setAttribute("class", "far fa-edit");
        iconElem.title= "Edit user";
        iconElem.onclick = function (){
            editUserDetails(id, usersList[id]);
        }
        tableCell.appendChild(iconElem);

        iconElem = document.createElement("i");
        iconElem.setAttribute("class", "fas fa-redo");
        iconElem.setAttribute("style", "margin-left: 10px;");
        iconElem.title= "Generate new Access URL";
        iconElem.onclick = function (){
            editUserRequest(id, usersList[id]["name"], usersList[id]["email"]);
        }
        tableCell.appendChild(iconElem);

        iconElem = document.createElement("i");
        iconElem.setAttribute("class", "fas fa-qrcode");
        iconElem.setAttribute("style", "margin-left: 10px;");
        iconElem.title= "Check QR code";
        iconElem.onclick = function () {
            buildModal("qrcode-modal");

            // fill user name
            if (usersList[id]["name"] === ""){
                document.getElementById("user-name-value").innerHTML = "User "+id.toString();
                document.getElementById("user-name-value").setAttribute("style", "font-style: italic");
            } else {
                document.getElementById("user-name-value").innerHTML = usersList[id]["name"].toString();
            }

            // fill e-mail
            document.getElementById("user-email-value").innerHTML = usersList[id]["email"].toString();

            let url = window.location.protocol + "//" + window.location.hostname + "/app/paging_users_frame.html?token="+usersList[id]["secret"].toString();

            // fill access token
            document.getElementById("user-token-value").innerHTML = url;

            //create QRCode
            document.getElementById("qrcode").innerHTML = "";
            new QRCode(document.getElementById("qrcode"), url); 

            //download user info
            let downloadBtn = document.getElementById("downloadUserInfo");
            downloadBtn.onclick = function () {
                var element = document.getElementById('qrcode-modal-body');
                let filename = "User_";
                if (usersList[id]["name"] === ""){
                    filename += id.toString();
                } else {
                    filename += usersList[id]["name"].toString();
                }
                filename += "_info.pdf";
                worker = html2pdf().from(element.innerHTML).save(filename);
            }
            
            showPopup("qrcode-modal",false, false);
            
        }
        tableCell.appendChild(iconElem);

    });
            
}

function renderUserDiv() {

    if (Object.keys(usersList).length === 0) {
        const noUsersDivElem = document.createElement("div");
        noUsersDivElem.setAttribute("class", "section2");
        noUsersDivElem.setAttribute("style", "font-size: 13px; font-style: italic; padding: 4px; color: #888888; text-align: center; margin-top: 10%;");
        noUsersDivElem.innerText = "0 users created";
        return noUsersDivElem;
    }

    // create table div with users
    const mainTable = document.createElement("table");
    mainTable.setAttribute("id", "users-table");

    return mainTable;

}

window.onload = async function () {
    const userDiv = document.getElementById('users-section');
    if (userDiv) {
        usersList = await getListOfUsers();
        const mediaInternal = renderUserDiv();
        userDiv.appendChild(mediaInternal);
        if (Object.keys(usersList).length !== 0) renderUsersTable();
        paginate('#users-table',N_ROWS);
    } else {
        console.error("ERROR! Element can't be deployed! Please, clear browser cache and reload.");
    }
}